local PitBull4 = PitBull4
if not PitBull4 then
	error("PitBull4_VengeanceBar requires PitBull4")
end

local floor = math.floor
local tanks = {}

local InTankStance
do
	local class = select(2, UnitClass("player"))
	local tankStances = {
		PALADIN = true,
		DEATHKNIGHT = 1,
		DRUID = 1, -- 5
		WARRIOR = 2, -- 18
		MONK = 1, -- 23
	}
	local classTankStance = tankStances[class]

	function InTankStance()
		return classTankStance == true or classTankStance == GetShapeshiftForm()
	end
end


local GetVengeanceAP
do
	local UnitBuff = UnitBuff
	local vengeance = GetSpellInfo(93098)
	function GetVengeanceAP(unit)
		local ap = select(15, UnitBuff(unit or "player", vengeance)) or 0
		return floor(ap)
	end
end

local GetVengeanceAPMax
do
	local UnitHealthMax = UnitHealthMax
	function GetVengeanceAPMax(unit)
		local hp = UnitHealthMax(unit or "player") or 0
		return hp -- floor(hp * 0.5)
	end
end

local PitBull4_VengeanceBar = PitBull4:NewModule("VengeanceBar", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("PitBull4_VengeanceBar")

PitBull4_VengeanceBar:SetModuleType("bar")
PitBull4_VengeanceBar:SetName(L["Vengeance bar"])
PitBull4_VengeanceBar:SetDescription(L["Show a bar indicating the attack power gained from Vengeance."])
PitBull4_VengeanceBar:SetDefaults({
	check_stance = false,
	position = 4,
})

function PitBull4_VengeanceBar:OnEnable()
	--Allow for getting the AP value in LuaTexts
	local LuaTexts = PitBull4:GetModule("LuaTexts", true)
	if LuaTexts then
		LuaTexts.ScriptEnv.VengeanceAP = GetVengeanceAP
		LuaTexts.ScriptEnv.VengeanceAPMax = GetVengeanceAPMax
	end

	self:RegisterEvent("PLAYER_ROLES_ASSIGNED")
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "PLAYER_ROLES_ASSIGNED")
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "PLAYER_ROLES_ASSIGNED")
	self:PLAYER_ROLES_ASSIGNED()
end

function PitBull4_VengeanceBar:OnDisable()
	wipe(tanks)
	self:UpdateAll()
end

function PitBull4_VengeanceBar:GetValue(frame)
	local unit = frame.unit
	if tanks[unit] then
		if unit == "player" and self:GetLayoutDB(frame).check_stance and not InTankStance() then
			return
		end

		local cur, max = GetVengeanceAP(unit), GetVengeanceAPMax(unit)
		if cur == 0 then
			return 0
		elseif max > 0 then
			--[[
			local _, _, diff = GetInstanceInfo()
			if diff == 4 or diff == 6 then
				max = max * 0.5
			else
				max = max * 0.3
			end
			--]]
			if cur > max then
				return 1
			else
				return cur / max
			end
		end
	end
end

function PitBull4_VengeanceBar:GetExampleValue(frame)
	if frame.unit == "player" then
		return 0.3
	end
end

function PitBull4_VengeanceBar:GetColor()
	local color = PitBull4.PowerColors.POWER_TYPE_HEAT
	if color then
		return color[1], color[2], color[3]
	end
end


function PitBull4_VengeanceBar:PLAYER_ROLES_ASSIGNED()
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("UNIT_MAXHEALTH")
	wipe(tanks)

	local spec = GetSpecialization()
	if spec and GetSpecializationRole(spec) == "TANK" then
		tanks.player = true
	end

	if IsInRaid() then
		for i=1, GetNumGroupMembers() do
			local unit = ("raid%d"):format(i)
			local role, _, combatRole = select(10, GetRaidRosterInfo(i))
			if (combatRole == "TANK" or role == "MAINTANK") then
				tanks[unit] = true
			end
		end
	elseif IsInGroup() then
		for i=1, GetNumSubgroupMembers() do
			local unit = ("party%d"):format(i)
			if UnitGroupRolesAssigned(unit) == "TANK" then
				tanks[unit] = true
			end
		end
	end

	if next(tanks) then
		self:RegisterEvent("UNIT_AURA")
		self:RegisterEvent("UNIT_MAXHEALTH")
	end

	self:UpdateAll()
end

function PitBull4_VengeanceBar:UNIT_AURA(_, unit)
	if tanks[unit] then
		self:UpdateForUnitID(unit)
	end
end

function PitBull4_VengeanceBar:UNIT_MAXHEALTH(_, unit)
	if tanks[unit] then
		local hp = UnitHealthMax(unit) or 0
		if hp > 0 then
			self:UpdateForUnitID(unit)
		end
	end
end

PitBull4_VengeanceBar:SetLayoutOptionsFunction(function(self)
	return "check_stance", {
		type = "toggle",
		name = L["Hide in non-tank stance"],
		desc = L["Hide the vengeance bar for Player if not in a tanking stance."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).check_stance
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).check_stance = value
			PitBull4_VengeanceBar:UpdateForUnitID("player")
		end,
	},
	"color_pvp_by_class", nil,
	"hostility_color", nil,
	"hostility_color_npcs", nil
end)

